import os
import subprocess
import tempfile


class Converter(object):
    def __init__(self, ffmpeg_bin):
        self.ffmpeg = ffmpeg_bin

    def convert(self, paths):
        save_to = self.rename_extension(self.joined_filename(paths), 'mp4')
        if os.path.exists(save_to):
            raise OSError("File {} exists".format(save_to))
        filelist = self.create_filelist(paths)
        with tempfile.NamedTemporaryFile() as f:
            f.write(filelist)
            f.seek(0)
            f.flush()
            print(filelist)
            self.convert_and_join(f.name, save_to)
        return save_to

    def create_filelist(self, paths):
        filelist = ''
        for f in paths:
            filelist += 'file \'{}\'\n'.format(os.path.abspath(f))
        return bytes(filelist, 'UTF-8')

    def joined_filename(self, paths):
        """
        Assumes the paths are in the form {str}_{num}.{ext} e.g. path_01.mp4,
        path_02.mp4 etc. and returns path.mp4. Takes the first filename base.
        """
        base, ext = self.remove_digits(paths[0])
        return base + ext

    def convert_and_join(self, filelist, save_to):
        subprocess.check_call([self.ffmpeg, '-f', 'concat', '-i', filelist,
                               '-c', 'copy', save_to])

    @staticmethod
    def remove_digits(path):
        base, ext = os.path.splitext(path)
        last_underscore = base.rindex('_')
        return base[:last_underscore], ext

    @staticmethod
    def rename_extension(path, extension):
        ext = os.path.splitext(path)[1].split('.')[-1]
        return path.replace(ext, extension)



import os
import re
import unicodedata
import logging

import requests
from progressbar import ProgressBar, Bar, FileTransferSpeed, Percentage

from seizure.lib.config import Config
from seizure.lib.video import Video


class Downloader(object):
    def __init__(self, vod: Video, config: Config):
        self.vod = vod
        self.config = config

    def download(self, quality=None, folder=None):
        quality = quality or self.vod.get_best_quality()
        folder = folder or self.default_folder()
        video_urls = self.vod.download_urls(quality)
        if not os.path.exists(folder):
            os.makedirs(folder)
        filenames = [os.path.join(folder, self.generate_filename(n))
                     for n, url in enumerate(video_urls)]
        for n, (v, f) in enumerate(zip(video_urls, filenames)):
            fraction = "{}/{}".format(n+1, len(filenames))
            if self.can_download_file(f):
                logging.info("Downloading {} {}".format(fraction, v))
                self.download_chunk(v, f)
            else:
                logging.info("Skipping {} {}".format(fraction, v))
        return filenames

    def default_folder(self):
        return self.sanitize('{}'.format(self.vod.title))

    def download_chunk(self, url, to):
        to = to or self.default_filename(url)
        response = requests.get(url, stream=True)
        response.raise_for_status()
        self.write_to_file(response, to)
        finished = self.config.all_finished
        finished.append(to)
        self.config.update('finished', finished)

    def default_filename(self, url):
        return url.split('/')[-1]

    def can_download_file(self, filename):
        if not os.path.exists(filename) and self.config.finished(filename):
            finished = self.config.all_finished
            finished.remove(filename)
            self.config.update('finished', finished)
            return True
        return not self.config.finished(filename)

    @staticmethod
    def write_to_file(response, to):
        widgets = [Bar(), Percentage(), ' ', FileTransferSpeed()]
        with ProgressBar(widgets=widgets,
                         maxval=int(response.headers['Content-Length'])) \
                as pbar:
            with open(to, 'wb') as f:
                for n, block in enumerate(response.iter_content(1024)):
                    if not block:
                        break
                    f.write(block)
                    pbar.update(n * 1024)

    @staticmethod
    def sanitize(value):
        value = unicodedata.normalize('NFKD', value). \
            encode('ascii', 'ignore').decode('ascii')
        value = re.sub('[^\w\s-]', '', value).strip().lower()
        return re.sub('[-\s]+', '-', value)

    def generate_filename(self, num):
        title = self.sanitize(self.vod.title)
        t = self.sanitize(self.vod.start_time)
        num = str(num).zfill(2)
        return "{}_{}_{}.{}".format(title, t, num, self.vod.extension)



import argparse
import logging

from seizure.lib.config import Config
from seizure.lib.conversion import Converter
from seizure.lib.download import Downloader
from seizure.lib.video import Video

logging.basicConfig(level=logging.INFO)


def main():
    parser = argparse.ArgumentParser(description='Process some integers.')
    parser.add_argument('code', metavar='code', type=str, help='VOD code')
    parser.add_argument('--config', metavar='config', type=str,
                        default='config.ini', help='Config file')
    args = parser.parse_args()
    config = Config(args.config)
    video = Video(args.code)
    downloader = Downloader(video, config)
    filenames = downloader.download(quality=config.quality,
                                    folder=config.download_folder)
    converter = Converter(config.ffmpeg_binary)
    converted = converter.convert(filenames)
    logging.info("Converted {}".format(converted))

if __name__ == '__main__':
    main()



from urllib.parse import urljoin

import requests


class Twitch(object):
    BASE_URL = 'https://api.twitch.tv/'
    MIME_TYPE = 'application/vnd.twitchtv.v2+json'
    QUALITIES = ['240p', '360p', '480p', 'live']

    @classmethod
    def request(cls, resource, **kwargs):
        headers = {'Accept': cls.MIME_TYPE}
        response = requests.get(urljoin(cls.BASE_URL, resource),
                                headers=headers, **kwargs)
        response.raise_for_status()
        return response.json()



import configparser
import json
import os
import logging


class Config(object):
    default_values = {
        'ffmpeg binary': 'ffmpeg',
        'download folder': '',
        'quality': 'live',
        }

    def __init__(self, filename):
        self.filename = filename
        self.config = configparser.ConfigParser()
        self.config.read(filename)
        if not os.path.exists(self.filename):
            self.write_initial()

    def finished(self, filename):
        """Has the filename been started?"""
        return filename in self.config['DEFAULT']['finished']

    def write_initial(self):
        logging.info('Writing initial config {}'.format(self.filename))
        self.config['DEFAULT'] = self.default_values
        with open(self.filename, 'w') as f:
            self.config.write(f)

    def update(self, key, value):
        self.config['DEFAULT'][key] = json.dumps(value)
        with open(self.filename, 'w') as f:
            self.config.write(f)

    @property
    def all_finished(self):
        return json.loads(self.config['DEFAULT']['finished'])

    @property
    def download_folder(self):
        return self.config['DEFAULT']['download folder']

    @property
    def ffmpeg_binary(self):
        return self.config['DEFAULT']['ffmpeg binary']

    @property
    def quality(self):
        return self.config['DEFAULT']['quality']



from setuptools import setup

setup(
    name='seizure',
    version='1.0.1',
    packages=['seizure', 'seizure.lib'],
    license='',
    author='Shane Drury',
    author_email='shane.r.drury@gmail.com',
    description='Download VODs from Twitch',
    entry_points={'console_scripts': ['seizure = seizure.scripts:main'], },
    keywords=['twitch', 'vod', 'download'],
    url='https://github.com/ShaneDrury/seizure',
    download_url='https://github.com/ShaneDrury/seizure/tarball/1.0.1',
    install_requires=['requests', 'progressbar'],
    )



from seizure.lib.twitch import Twitch


class Video(object):
    def __init__(self, code: str):
        self.code = code
        self.vod = Twitch.request("api/videos/a{}".format(self.code))
        self.info = Twitch.request(
            "kraken/videos/a{}?on_site=1".format(self.code)
        )

    def download_urls(self, quality):
        return [c['url'] for c in self.chunks[quality]]

    @property
    def chunks(self):
        return self.vod['chunks']

    @property
    def title(self):
        return self.info['title']

    @property
    def game(self):
        return self.info['game']

    @property
    def start_time(self):
        return self.info['recorded_at']

    @property
    def qualities(self):
        return self.chunks.keys()

    @property
    def extension(self):
        """Assume they're all the same"""
        url = self.chunks[self.get_best_quality()][0]['url']
        return url.split('.')[-1]

    def get_best_quality(self):
        return sorted(self.qualities,
                      key=lambda x: Twitch.QUALITIES.index(x))[-1]
